@@ -1,7 +1,9 @@
 from __future__ import with_statement
 
-import sys
 import copy
+from operator import isMappingType
+import os
+import sys
 from contextlib import contextmanager
 
 from fudge import Fake, patched_context
@@ -11,13 +13,11 @@ from fabric.decorators import hosts, roles, task
 from fabric.main import (get_hosts, parse_arguments, _merge, _escape_split,
         load_fabfile, list_commands, _task_names, _crawl, crawl,
         COMMANDS_HEADER, NESTED_REMINDER)
-
 import fabric.state
 from fabric.state import _AttributeDict
+from fabric.tasks import Task
 
 from utils import mock_streams, patched_env, eq_, FabricTest
-import os
-import sys
 
 
 #
@@ -173,8 +173,7 @@ def test_hosts_decorator_overrides_env_hosts_with_task_decorator_last():
     assert 'foo' not in get_hosts(command, [], [])
 
 
-@patched_env({'hosts': [' foo ', 'bar '], 'roles': [],
-        'exclude_hosts':[]})
+@patched_env({'hosts': [' foo ', 'bar '], 'roles': [], 'exclude_hosts': []})
 def test_hosts_stripped_env_hosts():
     """
     Make sure hosts defined in env.hosts are cleaned of extra spaces
@@ -225,6 +224,24 @@ def test_roles_decorator_expands_single_iterable():
     eq_(command.roles, role_list)
 
 
+#
+# Host exclusion
+#
+
+def dummy(): pass
+
+def test_get_hosts_excludes_cli_exclude_hosts_from_cli_hosts():
+    assert 'foo' not in get_hosts(dummy, ['foo', 'bar'], [], ['foo'])
+
+def test_get_hosts_excludes_cli_exclude_hosts_from_decorator_hosts():
+    assert 'foo' not in get_hosts(hosts('foo', 'bar')(dummy), [], [], ['foo'])
+
+@patched_env({'hosts': ['foo', 'bar'], 'exclude_hosts': ['foo']})
+def test_get_hosts_excludes_global_exclude_hosts_from_global_hosts():
+    assert 'foo' not in get_hosts(dummy, [], [], [])
+
+
+
 #
 # Basic role behavior
 #
@@ -418,6 +435,21 @@ def test_list_output():
         del list_output.description
 
 
+def name_to_task(name):
+    t = Task()
+    t.name = name
+    return t
+
+def strings_to_tasks(d):
+    ret = {}
+    for key, value in d.iteritems():
+        if isMappingType(value):
+            val = strings_to_tasks(value)
+        else:
+            val = name_to_task(value)
+        ret[key] = val
+    return ret
+
 def test_task_names():
     for desc, input_, output in (
         ('top level (single)', {'a': 5}, ['a']),
@@ -448,7 +480,7 @@ def test_task_names():
         ),
     ):
         eq_.description = "task name flattening: %s" % desc
-        yield eq_, _task_names(input_), output
+        yield eq_, _task_names(strings_to_tasks(input_)), output
         del eq_.description
 
 
@@ -462,3 +494,12 @@ def test_crawl():
         eq_.description = "crawling dotted names: %s" % desc
         yield eq_, _crawl(name, mapping), output
         del eq_.description
+
+
+def test_mapping_task_classes():
+    """
+    Task classes implementing the mapping interface shouldn't break --list
+    """
+    docstring, tasks = load_fabfile(fabfile('mapping'))
+    list_output('mapping', 'normal', COMMANDS_HEADER + """:\n
+    mapping_task""")
